"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniqueCellQueue = void 0;
const errors_1 = require("../errors");
/**
 * This queue will return each cell only once
 */
class UniqueCellQueue {
    queue = [];
    cellIds = [];
    /**
     * Adds a cell to the queue. If the given cell was in the queue already, it will not be added to the queue
     * @param cell The cell that should be added to the queue
     */
    add(cell) {
        if (this.cellIds.includes(cell.id))
            return;
        this.ignore(cell);
        this.queue.push(cell);
    }
    /**
     * Adds an array of cells to the queue. If a given cell was in the queue already, it will not be added to the queue
     * @param cells The cells that should be added to the queue
     */
    addAll(cells) {
        for (const cell of cells) {
            this.add(cell);
        }
    }
    /**
     * @param cell The cell that should never be added to the queue
     */
    ignore(cell) {
        this.cellIds.push(cell.id);
    }
    /**
     * @returns true when the queue is not empty
     */
    hasNext() {
        return this.queue.length > 0;
    }
    /**
     * @returns The next cell in the queue
     * @throws {QueueIsEmptyError} when the queue has no more cells
     */
    getNext() {
        const cell = this.queue.pop();
        if (!cell)
            throw new errors_1.QueueIsEmptyError();
        return cell;
    }
}
exports.UniqueCellQueue = UniqueCellQueue;
