import { GridEventDispatcher } from './utils';
/**
 * The row of a grid contains all information of the rows' cells
 */
export class Row {
    grid;
    row;
    id;
    _cells = [];
    eventDispatcher;
    /**
     * @param grid The grid this row is part of
     * @param row The row coordinate inside the grid
     */
    constructor(grid, row) {
        this.grid = grid;
        this.row = row;
        this.id = `row|${row}`;
        for (let col = 0; col < this.grid.width; col++) {
            this.cells.push(this.getCell(col));
        }
        this.eventDispatcher = new GridEventDispatcher();
        const forwardEvent = (event) => this.eventDispatcher.dispatchCellValueChangedEvent(event);
        for (const cell of this.cells) {
            cell.onValueChanged(forwardEvent);
        }
    }
    /**
     * @returns all cells of this row
     */
    get cells() {
        return this._cells;
    }
    /**
     * @param col The column coordinate of the grid
     * @returns A cell
     */
    getCell(col) {
        return this.grid.getCell({ row: this.row, col });
    }
    /**
     * @param callback A function that should be called, when a cell value of this row changes
     * @returns a function to unregister the callback
     */
    onCellValueChanged(callback) {
        return this.eventDispatcher.onCellValueChanged(callback);
    }
}
